@@ -226,8 +226,8 @@ static VipsIccInfo vips_icc_info_table[] = {
         { cmsSigGrayData, 1, TYPE_GRAY_8, TYPE_GRAY_16 },
 
         { cmsSigRgbData, 3, TYPE_RGB_8, TYPE_RGB_16 },
-        { cmsSigLabData, 3, TYPE_Lab_8, TYPE_Lab_16 },
-        { cmsSigXYZData, 3, -1, TYPE_XYZ_16 },
+        { cmsSigLabData, 3, TYPE_Lab_FLT, TYPE_Lab_16 },
+        { cmsSigXYZData, 3, TYPE_XYZ_FLT, TYPE_XYZ_16 },
 
         { cmsSigCmykData, 4, TYPE_CMYK_8, TYPE_CMYK_16 },
         { cmsSig4colorData, 4, TYPE_CMYK_8, TYPE_CMYK_16 },
@@ -308,14 +308,14 @@ vips_icc_build( VipsObject *object )
 			code->input_format = VIPS_FORMAT_FLOAT;
 			code->input_interpretation = 
 				VIPS_INTERPRETATION_LAB;
-			icc->in_icc_format = info->lcms_type16;
+			icc->in_icc_format = info->lcms_type8;
 			break;
 
 		case cmsSigXYZData:
 			code->input_format = VIPS_FORMAT_FLOAT;
 			code->input_interpretation = 
 				VIPS_INTERPRETATION_XYZ;
-			icc->in_icc_format = info->lcms_type16;
+			icc->in_icc_format = info->lcms_type8;
 			break;
 
 		case cmsSigCmykData:
@@ -545,6 +545,51 @@ vips_icc_print_profile( const char *name, cmsHPROFILE profile )
 }
 #endif /*DEBUG*/
 
+/* TRUE if the number of bands in the ICC profile is compatible with the
+ * image.
+ */
+static gboolean
+vips_image_is_profile_compatible( VipsImage *image, int profile_bands )
+{
+	switch( image->Type ) {
+		case VIPS_INTERPRETATION_B_W:
+		case VIPS_INTERPRETATION_GREY16:
+			/* The ICC profile needs to be monochrome.
+			 */
+			return( profile_bands == 1 );
+
+		case VIPS_INTERPRETATION_XYZ:
+		case VIPS_INTERPRETATION_LAB:
+		case VIPS_INTERPRETATION_LABQ:
+		case VIPS_INTERPRETATION_RGB:
+		case VIPS_INTERPRETATION_CMC:
+		case VIPS_INTERPRETATION_LCH:
+		case VIPS_INTERPRETATION_LABS:
+		case VIPS_INTERPRETATION_sRGB:
+		case VIPS_INTERPRETATION_YXY:
+		case VIPS_INTERPRETATION_RGB16:
+		case VIPS_INTERPRETATION_scRGB:
+		case VIPS_INTERPRETATION_HSV:
+			/* The band count in the ICC profile must correspond to that of
+			 * the image, with a maximum of 3 bands allowed.
+			 */
+			return( VIPS_MIN( 3, image->Bands ) == profile_bands );
+
+		case VIPS_INTERPRETATION_CMYK:
+			/* CMYK images can only be imported if the ICC-profile has at
+			 * least 4 bands thereby blocking the usage of RGB profiles.
+			 */
+			return( profile_bands >= 4 );
+
+		case VIPS_INTERPRETATION_MULTIBAND:
+		case VIPS_INTERPRETATION_HISTOGRAM:
+		case VIPS_INTERPRETATION_MATRIX:
+		case VIPS_INTERPRETATION_FOURIER:
+		default:
+			return( image->Bands >= profile_bands );
+	}
+}
+
 /* Load a profile from a blob and check compatibility with image, intent and
  * direction.
  *
@@ -583,17 +628,7 @@ vips_icc_load_profile_blob( VipsBlob *blob,
         }
 
 	if( image &&
-		image->Bands < info->bands ) { 
-		VIPS_FREEF( cmsCloseProfile, profile );
-		g_warning( "%s", _( "profile incompatible with image" ) );
-		return( NULL );
-	}
-
-	/* Spot the common error of an RGB profile and a CMYK image.
-	 */
-	if( image &&
-		image->Type == VIPS_INTERPRETATION_CMYK &&
-		info->bands == 3 ) {
+		!vips_image_is_profile_compatible( image, info->bands ) ) {
 		VIPS_FREEF( cmsCloseProfile, profile );
 		g_warning( "%s", _( "profile incompatible with image" ) );
 		return( NULL );
@@ -616,14 +651,13 @@ vips_icc_load_profile_blob( VipsBlob *blob,
  * unref the blob if it's useless.
  */
 static cmsHPROFILE
-vips_icc_verify_blob( VipsBlob **blob, 
-	VipsImage *image, VipsIntent intent, int direction )
+vips_icc_verify_blob( VipsBlob **blob, VipsImage *image, VipsIntent intent )
 {
 	if( *blob ) {
 		cmsHPROFILE profile;
 
 		if( !(profile = vips_icc_load_profile_blob( *blob, 
-			image, intent, direction )) ) {
+			image, intent, LCMS_USED_AS_INPUT )) ) {
 			vips_area_unref( (VipsArea *) *blob );
 			*blob = NULL;
 		}
@@ -663,7 +697,7 @@ vips_icc_set_import( VipsIcc *icc,
 		(embedded || !input_profile_filename) ) {
 		icc->in_blob = vips_icc_get_profile_image( code->in );
 		icc->in_profile = vips_icc_verify_blob( &icc->in_blob,
-			code->in, icc->intent, LCMS_USED_AS_INPUT );
+			code->in, icc->intent );
 	}
 
 	/* Try profile from filename.
@@ -675,7 +709,7 @@ vips_icc_set_import( VipsIcc *icc,
 			!vips_profile_load( input_profile_filename, 
 				&icc->in_blob, NULL ) &&
 			(icc->in_profile = vips_icc_verify_blob( &icc->in_blob, 
-				code->in, icc->intent, LCMS_USED_AS_INPUT )) )
+				code->in, icc->intent )) )
 			icc->non_standard_input_profile = TRUE;
 	}
 
@@ -683,13 +717,27 @@ vips_icc_set_import( VipsIcc *icc,
 	 */
 	if( code->in &&
 		!icc->in_profile ) {
-		const char *name = code->in->Type == VIPS_INTERPRETATION_CMYK ?
-			"cmyk" : "srgb";
+		const char *name;
+
+		switch( code->in->Type ) {
+		case VIPS_INTERPRETATION_B_W:
+		case VIPS_INTERPRETATION_GREY16:
+			name = "sgrey";
+			break;
+
+		case VIPS_INTERPRETATION_CMYK:
+			name = "cmyk";	
+			break;
+
+		default:
+			name = "srgb";
+			break;
+		}
 
 		if( 
 			!vips_profile_load( name, &icc->in_blob, NULL ) &&
 			(icc->in_profile = vips_icc_verify_blob( &icc->in_blob, 
-				code->in, icc->intent, LCMS_USED_AS_INPUT )) )
+				code->in, icc->intent )) )
 			icc->non_standard_input_profile = TRUE;
 	}
 
@@ -773,7 +821,7 @@ vips_icc_import_build( VipsObject *object )
 
 	if( icc->pcs == VIPS_PCS_LAB ) { 
 		cmsCIExyY white;
-		cmsWhitePointFromTemp( &white, 6500 );
+		cmsWhitePointFromTemp( &white, 6504 );
 
 		icc->out_profile = cmsCreateLab4Profile( &white );
 	}
@@ -802,44 +850,67 @@ vips_icc_import_build( VipsObject *object )
 	return( 0 );
 }
 
-static void 
+static void
 decode_lab( guint16 *fixed, float *lab, int n )
 {
 	int i;
 
-        for( i = 0; i < n; i++ ) {
-                lab[0] = (double) fixed[0] / 652.800;
-                lab[1] = ((double) fixed[1] / 256.0) - 128.0;
-                lab[2] = ((double) fixed[2] / 256.0) - 128.0;
+	for( i = 0; i < n; i++ ) {
+		/* cmsLabEncoded2Float inlined.
+		 */
+		lab[0] = (double) fixed[0] / 655.35;
+		lab[1] = ((double) fixed[1] / 257.0) - 128.0;
+		lab[2] = ((double) fixed[2] / 257.0) - 128.0;
 
-                lab += 3;
-                fixed += 3;
-        }
+		lab += 3;
+		fixed += 3;
+	}
 }
 
-#define X_FAC (VIPS_D50_X0 * 32768 / (VIPS_D65_X0 * 100))
-#define Y_FAC (VIPS_D50_Y0 * 32768 / (VIPS_D65_Y0 * 100))
-#define Z_FAC (VIPS_D50_Z0 * 32768 / (VIPS_D65_Z0 * 100))
+/* The matrix already includes the D65 channel weighting, so we just scale by
+ * Y.
+ */
+#define SCALE (VIPS_D65_Y0)
 
-static void 
+static void
 decode_xyz( guint16 *fixed, float *xyz, int n )
 {
 	int i;
 
-        for( i = 0; i < n; i++ ) {
-                xyz[0] = (double) fixed[0] / X_FAC;
-                xyz[1] = (double) fixed[1] / Y_FAC;
-                xyz[2] = (double) fixed[2] / Z_FAC;
+	for( i = 0; i < n; i++ ) {
+		/* cmsXYZEncoded2Float inlined.
+ 		 */
+		float X = fixed[0] / 32768.0;
+		float Y = fixed[1] / 32768.0;
+		float Z = fixed[2] / 32768.0;
 
-                xyz += 3;
-                fixed += 3;
-        }
+		X *= SCALE;
+		Y *= SCALE;
+		Z *= SCALE;
+
+		/* Transform XYZ D50 to D65, chromatic adaption is done with the
+		 * Bradford transformation.
+		 * See: https://fujiwaratko.sakura.ne.jp/infosci/colorspace/bradford_e.html
+		 */
+		xyz[0] = 0.955513 * X +
+			-0.023073 * Y +
+			0.063309 * Z;
+		xyz[1] = -0.028325 * X +
+			1.009942 * Y +
+			0.021055 * Z;
+		xyz[2] = 0.012329 * X +
+			-0.020536 * Y +
+			1.330714 * Z;
+
+		xyz += 3;
+		fixed += 3;
+	}
 }
 
 /* Process a buffer of data.
  */
 static void
-vips_icc_import_line( VipsColour *colour, 
+vips_icc_import_line( VipsColour *colour,
 	VipsPel *out, VipsPel **in, int width )
 {
 	VipsIcc *icc = (VipsIcc *) colour;
@@ -859,7 +930,7 @@ vips_icc_import_line( VipsColour *colour,
 
 		cmsDoTransform( icc->trans, p, encoded, chunk );
 
-		if( icc->pcs == VIPS_PCS_LAB ) 
+		if( icc->pcs == VIPS_PCS_LAB )
 			decode_lab( encoded, q, chunk );
 		else
 			decode_xyz( encoded, q, chunk );
@@ -936,7 +1007,7 @@ vips_icc_export_build( VipsObject *object )
 
 	if( icc->pcs == VIPS_PCS_LAB ) { 
 		cmsCIExyY white;
-		cmsWhitePointFromTemp( &white, 6500 );
+		cmsWhitePointFromTemp( &white, 6504 );
 
 		icc->in_profile = cmsCreateLab4Profile( &white );
 	}
@@ -968,77 +1039,59 @@ vips_icc_export_build( VipsObject *object )
 	return( 0 );
 }
 
-/* Pack a buffer of floats into lcms's fixed-point formats. Cut from
- * lcms-1.0.8.
- */
-static void 
-encode_lab( float *lab, guint16 *fixed, int n )
+static void
+encode_xyz( float *in, float *out, int n )
 {
 	int i;
 
 	for( i = 0; i < n; i++ ) {
-		float L = lab[0];
-		float a = lab[1];
-		float b = lab[2];
+		float X = in[0] / SCALE;
+		float Y = in[1] / SCALE;
+		float Z = in[2] / SCALE;
 
-		if( L < 0 ) 
-			L = 0;
-		if( L > 100. ) 
-			L = 100.;
+		/* Transform XYZ D65 to D50, chromatic adaption is done with the
+		 * Bradford transformation.
+		 * See: https://fujiwaratko.sakura.ne.jp/infosci/colorspace/bradford_e.html
+		 */
+		out[0] = 1.047886 * X +
+			0.022919 * Y +
+			-0.050216 * Z;
+		out[1] = 0.029582 * X +
+			0.990484 * Y +
+			-0.017079 * Z;
+		out[2] = -0.009252 * X +
+			0.015073 * Y +
+			0.751678 * Z;
 
-		if( a < -128. ) 
-			a = -128;
-		if( a > 127.9961 ) 
-			a = 127.9961;
-		if( b < -128. ) 
-			b = -128;
-		if( b > 127.9961 ) 
-			b = 127.9961;
-
-		fixed[0] = L *  652.800 + 0.5;
-		fixed[1] = (a + 128.0) * 256.0 + 0.5;
-		fixed[2] = (b + 128.0) * 256.0 + 0.5;
-
-		lab += 3;
-		fixed += 3;
+		in += 3;
+		out += 3;
 	}
 }
 
-#define MAX_ENCODEABLE_XYZ  (100 * (1.0 + 32767.0 / 32768.0))
-
-// 1.15 fixed point for XYZ
-
-static void 
-encode_xyz( float *xyz, guint16 *fixed, int n )
+static void
+vips_icc_export_line_xyz( VipsColour *colour,
+	VipsPel *out, VipsPel **in, int width )
 {
-	int i;
+	VipsIcc *icc = (VipsIcc *) colour;
 
-	for( i = 0; i < n; i++ ) {
-		float X = xyz[0];
-		float Y = xyz[1];
-		float Z = xyz[2];
+	float *p;
+	VipsPel *q;
+	int x;
 
-		if( X < 0 ) 
-			X = 0;
-		if( X > MAX_ENCODEABLE_XYZ ) 
-			X = MAX_ENCODEABLE_XYZ;
+	/* Buffer of encoded float pixels we transform.
+	 */
+	float encoded[3 * PIXEL_BUFFER_SIZE];
 
-		if( Y < 0 ) 
-			Y = 0;
-		if( Y > MAX_ENCODEABLE_XYZ ) 
-			Y = MAX_ENCODEABLE_XYZ;
+	p = (float *) in[0];
+	q = (VipsPel *) out;
+	for( x = 0; x < width; x += PIXEL_BUFFER_SIZE ) {
+		const int chunk = VIPS_MIN( width - x, PIXEL_BUFFER_SIZE );
 
-		if( Z < 0 ) 
-			Z = 0;
-		if( Z > MAX_ENCODEABLE_XYZ ) 
-			Z = MAX_ENCODEABLE_XYZ;
+		encode_xyz( p, encoded, chunk );
+		cmsDoTransform( icc->trans, encoded, q, chunk );
 
-		fixed[0] = X * X_FAC + 0.5;
-		fixed[1] = Y * Y_FAC + 0.5;
-		fixed[2] = Z * Z_FAC + 0.5;
-
-		xyz += 3;
-		fixed += 3;
+		p += PIXEL_BUFFER_SIZE * 3;
+		q += PIXEL_BUFFER_SIZE * VIPS_IMAGE_SIZEOF_PEL( colour->out );
 	}
 }
 
@@ -1050,29 +1103,10 @@ vips_icc_export_line( VipsColour *colour,
 {
 	VipsIcc *icc = (VipsIcc *) colour;
 
-	float *p;
-	VipsPel *q;
-	int x;
-
-	/* Buffer of encoded 16-bit pixels we transform.
-	 */
-	guint16 encoded[3 * PIXEL_BUFFER_SIZE];
-
-	p = (float *) in[0];
-	q = (VipsPel *) out;
-	for( x = 0; x < width; x += PIXEL_BUFFER_SIZE ) {
-		const int chunk = VIPS_MIN( width - x, PIXEL_BUFFER_SIZE );
-
-		if( icc->pcs == VIPS_PCS_LAB )
-			encode_lab( p, encoded, chunk );
-		else
-			encode_xyz( p, encoded, chunk );
-
-		cmsDoTransform( icc->trans, encoded, q, chunk );
-
-		p += PIXEL_BUFFER_SIZE * 3;
-		q += PIXEL_BUFFER_SIZE * VIPS_IMAGE_SIZEOF_PEL( colour->out );
-	}
+	if( icc->pcs == VIPS_PCS_LAB )
+		cmsDoTransform( icc->trans, in[0], out, width );
+	else
+		vips_icc_export_line_xyz( colour, out, in, width );
 }
 
 static void
@@ -1328,9 +1362,9 @@ vips_icc_is_compatible_profile( VipsImage *image,
                 return( FALSE );
         }
 
-	if( image->Bands < info->bands ) {
-                /* Too few bands,
-                 */
+	if( !vips_image_is_profile_compatible( image, info->bands ) ) {
+		/* Bands mismatch.
+		 */
 		VIPS_FREEF( cmsCloseProfile, profile );
 		return( FALSE );
 	}
@@ -1387,7 +1421,7 @@ vips_icc_is_compatible_profile( VipsImage *image,
  *
  * If @embedded is set, the input profile is taken from the input image
  * metadata. If there is no embedded profile,
- * @input_profile_filename is used as a fall-back. 
+ * @input_profile is used as a fall-back. 
  * You can test for the
  * presence of an embedded profile with
  * vips_image_get_typeof() with #VIPS_META_ICC_NAME as an argument. This will
