@@ -40,7 +40,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
-#include <vips/intl.h>
+#include <glib/gi18n-lib.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -470,6 +470,7 @@ vips_foreign_load_csv_class_init( VipsForeignLoadCsvClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsOperationClass *operation_class = VIPS_OPERATION_CLASS( class );
 	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
 
 	gobject_class->dispose = vips_foreign_load_csv_dispose;
@@ -480,6 +481,11 @@ vips_foreign_load_csv_class_init( VipsForeignLoadCsvClass *class )
 	object_class->description = _( "load csv" );
 	object_class->build = vips_foreign_load_csv_build;
 
+	/* This is fuzzed, but you're unlikely to want to use it on 
+	 * untrusted files.
+	 */
+	operation_class->flags |= VIPS_OPERATION_UNTRUSTED;
+
 	load_class->get_flags = vips_foreign_load_csv_get_flags;
 	load_class->header = vips_foreign_load_csv_header;
 	load_class->load = vips_foreign_load_csv_load;
@@ -651,7 +657,7 @@ vips_foreign_load_csv_source_class_init( VipsForeignLoadCsvFileClass *class )
 	object_class->nickname = "csvload_source";
 	object_class->build = vips_foreign_load_csv_source_build;
 
-	operation_class->flags = VIPS_OPERATION_NOCACHE;
+	operation_class->flags |= VIPS_OPERATION_NOCACHE;
 
 	load_class->is_a_source = vips_foreign_load_csv_source_is_a_source;
 
