@@ -2740,6 +2740,7 @@ vips_pdfload_source( VipsSource *source, VipsImage **out, ... )
  * * @associated: %gchararray, load this associated image
  * * @attach_associated: %gboolean, attach all associated images as metadata
  * * @autocrop: %gboolean, crop to image bounds
+ * * @rgb: %gboolean, output RGB (not RGBA) pixels
  *
  * Read a virtual slide supported by the OpenSlide library into a VIPS image.
  * OpenSlide supports images in Aperio, Hamamatsu, MIRAX, Sakura, Trestle,
@@ -2762,7 +2763,8 @@ vips_pdfload_source( VipsSource *source, VipsImage **out, ... )
  * are attached as "openslide-associated-XXXXX", where XXXXX is the name of the
  * associated image.
  *
- * The output of this operator is always RGBA.
+ * By default, the output of this operator is RGBA. Set @rgb to enable RGB
+ * output.
  *
  * See also: vips_image_new_from_file().
  *
@@ -2793,6 +2795,7 @@ vips_openslideload( const char *filename, VipsImage **out, ... )
  * * @associated: %gchararray, load this associated image
  * * @attach_associated: %gboolean, attach all associated images as metadata
  * * @autocrop: %gboolean, crop to image bounds
+ * * @rgb: %gboolean, output RGB (not RGBA) pixels
  *
  * Exactly as vips_openslideload(), but read from a source. 
  *
@@ -2833,6 +2836,7 @@ vips_foreign_operation_init( void )
 	extern GType vips_foreign_save_pgm_target_get_type( void );
 	extern GType vips_foreign_save_ppm_target_get_type( void ); 
 	extern GType vips_foreign_save_pfm_target_get_type( void ); 
+	extern GType vips_foreign_save_pnm_target_get_type( void ); 
 
 	extern GType vips_foreign_load_png_file_get_type( void ); 
 	extern GType vips_foreign_load_png_buffer_get_type( void ); 
@@ -2989,6 +2993,7 @@ vips_foreign_operation_init( void )
 	vips_foreign_save_pgm_target_get_type(); 
 	vips_foreign_save_ppm_target_get_type(); 
 	vips_foreign_save_pfm_target_get_type(); 
+	vips_foreign_save_pnm_target_get_type(); 
 #endif /*HAVE_PPM*/
 
 #ifdef HAVE_RADIANCE
@@ -3147,18 +3152,15 @@ vips_foreign_operation_init( void )
 	vips_foreign_save_nifti_get_type(); 
 #endif /*HAVE_NIFTI*/
 
-#if defined(HAVE_HEIF_DECODER) && !defined(HEIF_MODULE)
+#if defined(HAVE_HEIF) && !defined(HEIF_MODULE)
 	vips_foreign_load_heif_file_get_type(); 
 	vips_foreign_load_heif_buffer_get_type(); 
 	vips_foreign_load_heif_source_get_type(); 
-#endif /*defined(HAVE_HEIF_DECODER) && !defined(HEIF_MODULE)*/
-
-#if defined(HAVE_HEIF_ENCODER) && !defined(HEIF_MODULE)
 	vips_foreign_save_heif_file_get_type(); 
 	vips_foreign_save_heif_buffer_get_type(); 
 	vips_foreign_save_heif_target_get_type(); 
 	vips_foreign_save_avif_target_get_type();
-#endif /*defined(HAVE_HEIF_ENCODER) && !defined(HEIF_MODULE)*/
+#endif /*defined(HAVE_HEIF) && !defined(HEIF_MODULE)*/
 
 	vips__foreign_load_operation = 
 		g_quark_from_static_string( "vips-foreign-load-operation" ); 
