@@ -229,7 +229,11 @@ vips_foreign_save_jxl_build( VipsObject *object )
 	VipsForeignSaveJxl *jxl = (VipsForeignSaveJxl *) object;
 	VipsImage **t = (VipsImage **) vips_object_local_array( object, 5 );
 
-	JxlEncoderOptions *options;
+#ifdef HAVE_LIBJXL_0_7
+	JxlEncoderFrameSettings *frame_settings;
+#else
+	JxlEncoderOptions *frame_settings;
+#endif
 	JxlEncoderStatus status;
 	VipsImage *in;
 	VipsBandFormat format;
@@ -245,7 +249,7 @@ vips_foreign_save_jxl_build( VipsObject *object )
 	if( !vips_object_argument_isset( object, "distance" ) ) 
 		jxl->distance = jxl->Q >= 30 ?
 			0.1 + (100 - jxl->Q) * 0.09 :
-			6.4 + pow(2.5, (30 - jxl->Q) / 5.0f) / 6.25f;
+			53.0 / 3000.0 * jxl->Q * jxl->Q - 23.0 / 20.0 * jxl->Q + 25.0;
 
 	/* Distance 0 is lossless. libjxl will fail for lossy distance 0.
 	 */
@@ -280,9 +284,7 @@ vips_foreign_save_jxl_build( VipsObject *object )
 		return( -1 );
 	in = t[0];
 
-#ifdef HAVE_LIBJXL_JXLENCODERINITBASICINFO
 	JxlEncoderInitBasicInfo( &jxl->info );
-#endif
 
 	switch( in->BandFmt ) {
 	case VIPS_FORMAT_UCHAR:
@@ -415,29 +417,44 @@ vips_foreign_save_jxl_build( VipsObject *object )
 	if( vips_image_wio_input( in ) )
 		return( -1 );
 
-	options = JxlEncoderOptionsCreate( jxl->encoder, NULL );
-	JxlEncoderOptionsSetDecodingSpeed( options, jxl->tier );
-	JxlEncoderOptionsSetDistance( options, jxl->distance );
-	JxlEncoderOptionsSetEffort( options, jxl->effort );
-	JxlEncoderOptionsSetLossless( options, jxl->lossless );
+#ifdef HAVE_LIBJXL_0_7
+	frame_settings = JxlEncoderFrameSettingsCreate( jxl->encoder, NULL );
+	JxlEncoderFrameSettingsSetOption( frame_settings, 
+		JXL_ENC_FRAME_SETTING_DECODING_SPEED, jxl->tier );
+	JxlEncoderSetFrameDistance( frame_settings, jxl->distance );
+	JxlEncoderFrameSettingsSetOption( frame_settings, 
+		JXL_ENC_FRAME_SETTING_EFFORT, jxl->effort );
+	JxlEncoderSetFrameLossless( frame_settings, jxl->lossless );
+#else
+	frame_settings = JxlEncoderOptionsCreate( jxl->encoder, NULL );
+	JxlEncoderOptionsSetDecodingSpeed( frame_settings, jxl->tier );
+	JxlEncoderOptionsSetDistance( frame_settings, jxl->distance );
+	JxlEncoderOptionsSetEffort( frame_settings, jxl->effort );
+	JxlEncoderOptionsSetLossless( frame_settings, jxl->lossless );
+#endif
 
 #ifdef DEBUG
 	vips_foreign_save_jxl_print_info( &jxl->info );
 	vips_foreign_save_jxl_print_format( &jxl->format );
-	printf( "JxlEncoderOptions:\n" );
+	printf( "JxlEncoderFrameSettings:\n" );
 	printf( "    tier = %d\n", jxl->tier );
 	printf( "    distance = %g\n", jxl->distance );
 	printf( "    effort = %d\n", jxl->effort );
 	printf( "    lossless = %d\n", jxl->lossless );
 #endif /*DEBUG*/
 
-	if( JxlEncoderAddImageFrame( options, &jxl->format, 
+	if( JxlEncoderAddImageFrame( frame_settings, &jxl->format, 
 		VIPS_IMAGE_ADDR( in, 0, 0 ),
 		VIPS_IMAGE_SIZEOF_IMAGE( in ) ) ) { 
 		vips_foreign_save_jxl_error( jxl, "JxlEncoderAddImageFrame" );
 		return( -1 );
 	}
 
+	/* This function must be called after the final frame and/or box,
+	 * otherwise the codestream will not be encoded correctly.
+	 */
+	JxlEncoderCloseInput( jxl->encoder );
+
 	do {
 		uint8_t *out;
 		size_t avail_out;
@@ -479,9 +496,9 @@ vips_foreign_save_jxl_build( VipsObject *object )
 
 /* Type promotion for save ... unsigned ints + float + double.
  */
-static int bandfmt_jxl[10] = {
-     /* UC   C  US   S  UI   I  F  X  D DX */
-	UC, UC, US, US,  F,  F, F, F, F, F
+static VipsBandFormat bandfmt_jxl[10] = {
+	/* Band format:  UC  C   US  S   UI I  F  X  D DX */
+	/* Promotion: */ UC, UC, US, US, F, F, F, F, F, F
 };
 
 static void
@@ -525,7 +542,7 @@ vips_foreign_save_jxl_class_init( VipsForeignSaveJxlClass *class )
 		_( "Target butteraugli distance" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSaveJxl, distance ),
-		0, 15, 1.0 );
+		0.0, 25.0, 1.0 );
 
 	VIPS_ARG_INT( class, "effort", 12, 
 		_( "Effort" ), 
